from django.core.exceptions import ValidationError

import pytest

from .factories import ActiviteStructureFactory, StructureFactory

CNV = 0  # conventionnel
PRJ = 1  # projet bio
BIO = 2  # bio


class TestActiviteStructure:
    @pytest.mark.parametrize(
        ('nom', 'precision', 'exp_fail'),
        [
            ('autre', '', True),
            ('autre', 'whatever', False),
            ('lapin', '', False),
            ('lapin', 'whatever', False),
        ],
    )
    def test_validation(self, nom, precision, exp_fail):
        act_str = ActiviteStructureFactory.build(nom=nom, precision=precision)
        val_err = False
        exp_err_code = 'mandatory_field'

        try:
            act_str.clean()
        except ValidationError as err:
            val_err = err
        assert bool(val_err) == exp_fail
        if exp_fail:
            assert val_err.args[0]['precision'].code == exp_err_code

    @pytest.mark.parametrize(
        ('nom', 'precision', 'bio', 'exp_str'),
        [
            ('lapin', '', CNV, "[C] Lapin"),
            ('lapin', '', PRJ, "[P] Lapin"),
            ('lapin', '', BIO, "[B] Lapin"),
            ('autre', 'whatever', CNV, "[C] autre(whatever)"),
        ],
    )
    def test_str(self, nom, precision, bio, exp_str):
        act_str = ActiviteStructureFactory.build(
            nom=nom, precision=precision, activite_bio=bio
        )
        assert str(act_str) == exp_str


@pytest.mark.django_db
class TestStructure:
    @pytest.mark.parametrize(
        ('act_bio_list', 'ex_pri', 'ex_pri_bio', 'ex_annexe'),
        [
            ([], None, None, []),  # cas vide
            ([BIO, CNV, CNV], 0, 0, [1, 2]),
            ([CNV, BIO, CNV], 0, 1, [2]),
            ([CNV, CNV, BIO, BIO], 0, 2, [1, 3]),
            ([CNV, CNV, BIO, CNV, BIO], 0, 2, [1, 3, 4]),
        ],
    )
    def test__get_activite__head_bio(
        self, act_bio_list, ex_pri, ex_pri_bio, ex_annexe
    ):
        struct = StructureFactory()
        act_list = [
            ActiviteStructureFactory(structure=struct, activite_bio=is_bio)
            for is_bio in act_bio_list
        ]
        ex_pri = ex_pri if ex_pri is None else act_list[ex_pri]
        ex_pri_bio = ex_pri_bio if ex_pri_bio is None else act_list[ex_pri_bio]
        ex_annexe = [act_list[elm] for elm in ex_annexe]
        struct._get_activite()
        assert struct._act_pri == ex_pri
        assert struct._act_pri_bio == ex_pri_bio
        assert struct._act_annexe == ex_annexe

    def test_get_activite_integration(self):
        struct = StructureFactory()
        assert struct.activite_principale() is None
        assert struct.activite_principale_bio() is None
        assert struct.activites_annexes_list() == []
        assert struct.activites_annexes_bio_list() == []
        assert struct.activites_annexes() == '<ul></ul>'
